;; This program allows interaction between human and bot to play the glove game
;; It prints out the resultant coalition and comparison between the human paper and the bot
;; ver 0.0 2020

globals [myNumAgents mySelection myEnd myErrorCheck boolOtherSideCheck lsmycolor myround]
breed [myPlayers myPlayer]
breed [myLeftGloves myLeftGlove]
breed [myRightGloves myRightGlove]
myPlayers-own [intLeftGloves intRightGloves dblCurrentValue intMyCoalition boolSelected]

to setup
  clear-all
  file-close-all
  ;;********* Some inputs to change *******

  let lsLeftGlove MyLeftInput(GameNumber)
  let lsRightGlove MyRightInput(GameNumber)
  set myNumAgents min (list (length lsLeftGlove) (length lsRightGlove))
  set myErrorCheck FALSE ;; Will print out errors in game
  set boolOtherSideCheck FALSE

  set mySelection FALSE ;; Used to show player made selection
  set myEnd FALSE ;; Used to show user is finished
  set myRound 0

  set lsmycolor [gray brown magenta black orange turquoise lime 43 pink blue red]

  clear-output
  reset-ticks
  ask patches [ set pcolor white]

  ;; Create the agent
  MakeMyAgents lsLeftGlove lsRightGlove
  CurrentCoalitionValue

  MySetupOutput ;; sets up output file
  Print RunID

  print ""
  print "*** Human Players Turn ***"
end

to go
  ;; There are two stages each round: Player choose which members to join their team which is tested by user, then algorithm runs with possible question on players
  ;; First, we wait until the player press something
  let l ""
  let boolNeedPrint false

  ifelse (not (mySelection OR AutoRun))[
    ;; -------------------------------------------------------------------------------------------
    ;; STAGE 1: player selections coalition
    ;; -------------------------------------------------------------------------------------------
    if mouse-down?[WhenClickOnAgent]
    if myEnd [ ;; if user presses on Finish button
      user-message "Thanks for playing"
      MyFinalOutPutline
      file-close-all
      stop
    ]
  ][ ;; If human player presss "happy with selection"
     set myRound (myRound + 1)
    let boolOkFormNew false
    ifelse not AutoRun [
      ;; Human player selection
      ;; if none selected, assume that happy with current
      if (count myPlayers with [boolSelected] > 0)[
      ;; STEP 1a-0: Make sure agent player is selected
        ask myPlayer 0 [
          set shape "check"
          set boolSelected TRUE
        ]

        ;; STEP 1a-1: Make sure update coalition
        ;;CurrentCoalitionValue ;;This is redundent because do this after every update

        ;; STEP 1a-2: Find the value of suggested coalition
        ;; STEP 1a-3: Compare value to existing

        ;; Check coalition does not already exist
        ifelse MySuggestionAlreadyExist [
          print "Warning: Your selected Coalition Already exists"
        ][
          set boolOkFormNew AskBotsIfWantMove SelectedCoalitionValue 1

          set l MyOutPutlinePart1 TRUE FALSE ;; players turn and so did not say yes

          ;; STEP 1a-4a: If acceptable to all bots, then update to new coaltion
          ifelse boolOkFormNew [
            MyUpdateNewCoalition
            CurrentCoalitionValue
          ][
            ;; STEP 1a-4B: If not acceptable, lets player that is the case
            user-message (word "Not all the bots are happy with you suggestion so your suggested coalition did not form")
          ]
        ]
        MyOutPutlinePart2 l

        ;; Reset the players
        ask myPlayers [
          set boolSelected FALSE
          set shape "circle"
        ]
      ]]
      [
      ;; STEP 1b-0: Autorun - Select coalition suggestion type
       let c 0
       while [Not [boolSelected] of myPlayer 0][
        ask myPlayers [
          set boolSelected FALSE
          set shape "circle"
        ]
        set c one-of (list 0 2 3 4 6)
        botSelectionOfAgents c
       ]
       CurrentCoalitionValue ;; make sure values up to date

       let v SelectedCoalitionValue
       set boolOkFormNew AskBotsIfWantMove v 0

      ;; so player turn and whether accepte

      set l MyOutPutlinePart1 TRUE (v >  ([dblCurrentValue] of myPlayer 0))  ;; players turn and so did not say yes

       ;; STEP 2-4b: If all members are ok, update coalition
       if boolOkFormNew [
         MyUpdateNewCoalition
         CurrentCoalitionValue
       ]

      MyOutputLinePart2 l

       ;; need to check on other side
       ;; Not doing other side for '4' because Player 0 would not make suggestion if not going to move


       ask myPlayers [
         set boolSelected FALSE
         set shape "circle"
        ]
      ]

    ;; -------------------------------------------------------------------------------------------
    ;; STAGE 2a: Now bots get to select
    ;; -------------------------------------------------------------------------------------------
    print "*** Bot Players Turn ***"
    ;; STEP 2-1: Iterate through the different coalition methods
    foreach range 7 [i ->

      ;; STEP 2-2: Find suggested coalition
      botSelectionOfAgents i

      ;; need to ensure more that player agent involved
      ifelse OnlyPlayerAgentSelected
      [
        ;; this would mean that only playeragent selected
       ;; ask myplayer 1 [SelectBot]
        MyErrorPrint("Only player 0 selected so ignored request")
        MyErrorPrint(word MyCurrentCoalitionsOutput MyCurrentSuggestionOutput)
      ][

        ;; STEP 2-3: Find value of suggested coalition
        ;; STEP 2-4: Are bots in suggested coalition ok with new value?
        CurrentCoalitionValue
        let v SelectedCoalitionValue

        ifelse autorun [

          set boolOkFormNew AskBotsIfWantMove v 1 ;; ask all the other bots first
          ;; Seems i want to do this even in autorun

          ;; only check player one IF selected
          ;; NEW
          if ([boolSelected] of myplayer 0) AND boolOkFormNew [
            let b (v >  ([dblCurrentValue] of myPlayer 0))

            set l MyOutPutlinePart1 FALSE b  ;;
            set boolNeedPrint true
            if not b [set boolOkFormNew FALSE] ;; now if player 0 not wan to join, does not happen
          ]

        ][
          ;; IF human playing
          set boolOkFormNew AskBotsIfWantMove v 1

          ;; STEP 2-4a: if others ok, and if player involved, they ok with new coalition?
          if boolOkFormNew AND ([boolSelected] of myplayer 0) [
           set boolOkFormNew AskIfWhatJoin

            set l MyOutPutlinePart1 FALSE boolOkFormNew ;; players turn and so did not say yes
            set boolNeedPrint true
          ]
        ]

        ;; STEP 2-4b: If all members are ok, update coalition
        if boolOkFormNew [
          MyUpdateNewCoalition
          CurrentCoalitionValue
        ]
        if boolNeedPrint [MyOutPutlinePart2 l]
        set boolNeedPrint false

      ]

      ;; STEP 2-5: Remove suggested coalition and reset
      ask myPlayers [
        ;; deals with problem of need check both sides of coalition
        ;; Basically, next one in list of checks is checking the other side
        if(not boolOtherSideCheck) [set boolSelected FALSE]
        set shape "circle"
      ]


      set mySelection false ;; So that ready for next turn
      set myEnd FALSE;; just incase they pressed finish during the bots turn
      ;; CurrentCoalitionValue
    ];; for foreach loop

    print "*** Human Players Turn ***"
  ]

  if AutoRun [
    if((MyCSInCore >= 0)OR (ticks > AutoRunNumber))[
      MyFinalOutPutline
      file-close-all
      stop
    ]
  ]

  tick
end

;;**********************************************************************
;; Setup Agents
;;**********************************************************************
to MakeMyAgents [lsLeftGlove lsRightGlove]
   create-myPlayers  myNumAgents [
    fd 10
    set shape "circle"
    set label who
    set label-color 49

    set intLeftGloves (item who lsLeftGlove)
    set intRightGloves (item who lsRightGlove)
    set boolSelected FALSE
    set intMyCoalition who
    ;; need to update colors based on
    MyAgentColor
    ;; This put the glove graphics in place
    AgentLocation
    GloveLocation
  ]
end

to AgentLocation
  ;; turtle owned function
  set xcor (2 + ((who)*(max-pxcor - 3)/(myNumAgents - 1)))
  set ycor (max-pycor * 3 / 4)
end

to GloveLocation
  ;; turtle owned function
  let i 1
  hatch-myLeftGloves  intLeftGloves[
    set label ""
    set shape "left glove"
    set color red
    set xcor ([xcor] of myself - 1)
    set ycor ([ycor] of myself - i)
    face myself
    set i (i + 1)
  ]
  set i 1
  hatch-myRightGloves  intRightGloves[
    set label ""
    set shape "right glove"
    set color blue
    set xcor ([xcor] of myself + 1)
    set ycor ([ycor] of myself - i)
    face myself
    set i (i + 1)
  ]
end

to MyAgentColor
  ;; turtle owned function
  ;; Goes up to 13
  ifelse intMyCoalition > 9 [
    set color  (intMyCoalition * 21) + 1
  ][
    set color  item intMyCoalition lsmycolor
  ]
end

;;**********************************************************************
;; Selecting Agent
;;**********************************************************************
to WhenClickOnAgent
  ;; Observer owned
  ;; This occurs when a player is clicked on human player's turn
        ask myPlayers-on patch mouse-xcor mouse-ycor [
        ifelse not boolSelected[
          set shape "check"
          set boolSelected TRUE
        ][
          set shape "circle"
          set boolSelected FALSE
        ]
      ]
      wait 0.5 ;; because would unselect
End

to botSelectionOfAgents [n]
   ;; NEED NETLOGO 6.1+ to works
  ;;(1) Join coalitions; (2) Exit coalition; (3) Create pair coalition; (4) Defect coalition; (5) Split coalition; and (6) Return to an individual coalition.
  (ifelse
     (n = 0) [
      ;; Join coalition
      botTestJoinCoalition
    ]
   (n = 1) [botTestKickOut]
   (n = 2) [botTestPair]
   (n = 3) [botTestDefect]
    (n = 4) [botTestSplit]
    (n = 5) [botTestSplitOther]
    (n = 6) [botTestIndividual]
  [botSelectionRandom
  MyErrorPrint "Error: should be making an actual selection"
  ])
end

to botTestJoinCoalition
;; Join coalition
;;Two agents from different coalitions are chosen randomly. The payoffs of the merged coalitions are calculated.
;; If the payoffs improve for all members of both coalitions, a new coalition is formed, which is the merged coalition.
  ask one-of myPlayers [ SelectBot
    ;; puts all in current in same
    ask myPlayers with [intMyCoalition = [intMyCoalition] of myself][ selectbot]

    ;; need to check more than one coalition
    if (any? myPlayers with [not (intMyCoalition = [intMyCoalition] of myself)])[
      ask one-of myPlayers with [not (intMyCoalition = [intMyCoalition] of myself)][
        SelectBot
        ask myPlayers with [intMyCoalition = [intMyCoalition] of myself][ selectbot]
      ]
     ]
   ]
      ;;if myErrorCheck [ user-message (Word "is a join selected: " MyCurrentSuggestionOutput)]
end

to botTestKickOut
;;  Exit coalition
;; An agent from a coalition whose size is greater than one, i.e., not a singleton coalition, is randomly selected
;; The payoff of the coalition minus the agent is calculated. If all agents in the remaining coalition improve their payoff by removing the selected agent, the agent is removed from the coalition and forms a singleton coalition.
  let l[]
  foreach (range 0  myNumAgents)[i ->
    ask myplayer i [set l lput intMyCoalition l ]
  ]
  set l shuffle (remove-duplicates l)
  let n 0

  foreach l [ i ->
    set n (count myPlayers with [  intMyCoalition = i])
    if (n > 1)[
      ask n-of (n - 1) myPlayers with [intMyCoalition = i][selectbot]
;;   if myErrorCheck [ user-message (Word "is a kickout selected: " MyCurrentSuggestionOutput)]
      stop
    ]
  ]
  ;; if myErrorCheck [ user-message (Word "is a kickout selected: " MyCurrentSuggestionOutput)]
end

to botTestPair
;;Create a pair coalition
;;Two agents are randomly selected. The payoff for the coalition containing just both agents  is calculated. If the payoff of both agents is improved in this new coalition, both agents exit the current coalition in favor of the new one.
  ask n-of 2 myPlayers [selectbot]
  ;; if myErrorCheck [ user-message (Word "is a pair selected: " MyCurrentSuggestionOutput)]
end

to botTestDefect
;; Defect coalition
;; A randomly chosen agent selects a coalition to which he does not belong. If joining this coalition improves his payoff and the payoff of all members of the coalition, the agent defects from his current coalition and joins the new coalition.
  ask one-of myPlayers [
    SelectBot
     if (any? myPlayers with [not (intMyCoalition = [intMyCoalition] of myself)])[
      ask one-of myPlayers with [not (intMyCoalition = [intMyCoalition] of myself)][
        SelectBot
        ask myPlayers with [intMyCoalition = [intMyCoalition] of myself][ selectbot]
      ]
     ]
  ]
  ;; if myErrorCheck [ user-message (Word "is defect selected: " MyCurrentSuggestionOutput)]
End

to botTestSplit
;;Split coalition
;;A coalition is randomly chosen, and a random subset of agents from the coalition are selected to form a separate coalition.
;; If the members of the new coalition improve their payoff
;; OR the coalition that remained improve their payoff, the coalition splits into the two coalitions.
  ask one-of myPlayers [
    SelectBot
    let n count (myPlayers with [(intMyCoalition = [intMyCoalition] of myself)])
    if n > 2[
      set n random (n - 1)
      if (n > 0)[
        ask n-of n myPlayers with [(intMyCoalition = [intMyCoalition] of myself)][
          SelectBot
        ]
    ]]
  ]
  set boolOtherSideCheck true
  ;;  if myErrorCheck [ user-message (Word "is split selected: " MyCurrentSuggestionOutput)]
end

to botTestSplitOther
 ;; What happens if already split?
  ask one-of myPlayers with [boolSelected][
    ;; then there are no other players
    ask myPlayers with [(intMyCoalition = [intMyCoalition] of myself)][
     set boolSelected (not boolSelected)
    ]

  ]
  set boolOtherSideCheck false
   ;;   if myErrorCheck [ user-message (Word "is other split selected: " MyCurrentSuggestionOutput)]
end

 to botTestIndividual
;;Return to an individual coalition
;;An agent is randomly chosen. If that agent would be better off on their own, i.e., they prefer the singleton coalition to their current coalition;
;; then, they leave their current coalition to form the singleton coalition.
  ask one-of myPlayers with [who > 0] [SelectBot]
  ;;  if myErrorCheck [ user-message (Word "is single selected: " MyCurrentSuggestionOutput)]
end

to botSelectionRandom
;; Creates a random subgroup
  ask myplayers [if (1 > random 2) [SelectBot]]
end

to SelectBot
;; bot owned
;; Updates the bots value
        set boolSelected TRUE
        set shape "square"
end

;;**********************************************************************
;; Coaltion Value
;;**********************************************************************
to-report SelectedCoalitionValue
  ;; Observer owned
  ifelse any? myplayers with [boolselected][
    report (min list (sum [intLeftGloves] of myplayers with [boolselected]) (sum [intRightGloves] of myplayers with [boolselected])) / (count myplayers with [boolselected])
  ][
    ;; So none selected
    MyErrorPrint ("Warning: No agents were selected")
    report 0
  ]
end

to CurrentCoalitionValue
  ;; Observer owned
  ;; Works out the value of each coalition and updates that coalition
  foreach (range 0  (1 + max [intMyCoalition] of myPlayers)) [ c ->
    MyCoalitionValue c
  ]
end

to MyCoalitionValue [c]
  ;; Observer owned
  if any? myplayers with [intMyCoalition = c][
      let v ((min list (sum [intLeftGloves] of myplayers with [intMyCoalition = c]) (sum [intRightGloves] of myplayers with [intMyCoalition = c])) / (count myplayers with [intMyCoalition = c]))
      ask myplayers with [intMyCoalition = c] [set dblCurrentValue v]
  ]
end

;;**********************************************************************
;; Checking not selected existing coalition
;;**********************************************************************
to-report MySuggestionAlreadyExist
  ;; checks to make sure that suggested coalition does not already exist
  if any? myplayers with [boolselected][
  ;; determing coaltion of one member
    let intTestCoalition -1
    let myBool true
    ask one-of myplayers with [boolselected][set intTestCoalition intmycoalition]

    ;; Check all selected in same coalition
    ask myplayers with [boolselected][
      if not (intmycoalition = intTestCoalition)[set myBool false]
    ]

    ;; Check all not selected not in coalition
    ask myplayers with [not boolselected][
      if (intmycoalition = intTestCoalition)[set myBool false]
    ]

    report myBool
  ]

  report true
end

to-report OnlyPlayerAgentSelected
 ;; Checks to make sure that only human player is suggested
  if ([boolselected] of myplayer 0)[ ;;if agentzero selected
    if any? myplayers with [boolselected AND (who > 0)] [report false]
    report true
  ]

  ;;((count myplayers with [boolSelected]) < 2) AND ([boolSelected] of myplayer 0)
  report false
end


;;**********************************************************************
;; Updating Coalitions
;;**********************************************************************
to MyUpdateNewCoalition
  ;; Update the numbers of those selected
  UpdateCoalitionNumbering

  ;; updates numbers of standardize
  CurrentCoalitionsUpdate
  ask myplayers [MyAgentColor]
  ;; Update the value
  ;; Do we need to care?

end

;; Only going to change number of coalitions when necessary
to UpdateCoalitionNumbering
  ;; so remove current number and replace with max
  let n myNumAgents
  if (any? (myplayers with [boolSelected]))[
    ;; Set there value to one bigger than current
        ;; need to update zero coalition if contains player
    ifelse ([boolSelected] of myplayer 0)[
      set n FindSmallestEmptyCoalition
      ask myplayers with [intMyCoalition = 0][set intMyCoalition n]
      ask myplayers with [boolSelected] [set intMyCoalition 0]
    ][
     ;; So update does not include zero agent
      set n FindSmallestEmptyCoalition
      ask myplayers with [boolSelected] [set intMyCoalition n]
    ]
  ]
end


to-report FindSmallestEmptyCoalition
  ;; Note that if player
  let m myNumAgents
  let i 0

  While [i < myNumAgents][
   ifelse (any? myplayers with [intMyCoalition = i]) [
     set i (i + 1)
   ][
     set m i
     set i myNumAgents
   ]
  ]
   report m
end

to-report AskBotsIfWantMove [v i]
   let b TRUE
   ask myplayers with [boolSelected][
      if who >= i [ ;; so not human player
        if v <=  dblCurrentValue [
         set b FALSE
         print (word "Player " who " does not want to join coalition: " WhoAreCurrentSelection)
         stop
       ]
      ]
    ]
  report b
end





;;**********************************************************************
;; Interface
;;**********************************************************************
to-report WhoAreCurrentAskers
  let myAskers ""
  ifelse (count myPlayers with [boolSelected] > 2)[
     ;; Means that more than one other agent
     let l [who] of myPlayers with [boolSelected]
     set l but-first sort l
     set myAskers "Players "
     foreach but-last l [ x -> set myAskers (word myAskers x ", ")]
     set myAskers (word myAskers "and " last l)
  ][
     ;; Means one agent
     set myAskers "Player "
     set myAskers (word  myAskers max [who] of myPlayers with [boolSelected])
   ]
  report myAskers
end

to-report AskIfWhatJoin
      ;; Asking agent question
      let myQuestion (word whoarecurrentaskers " would like to form a coaltion with you. Will you join?")
            ifelse user-yes-or-no? myQuestion [report TRUE][report FALSE]
End

to-report WhoAreCurrentSelection
  let myAskers ""
  ifelse ((count myPlayers with [boolSelected]) > 1)[
     ;; Means that more than one other agent
     let l sort ([who] of myPlayers with [boolSelected])

     set myAskers "Players "
     foreach but-last l [ x -> set myAskers (word myAskers x ", ")]
     set myAskers (word myAskers "and " last l)
  ][
     ;; Means one agent
    ifelse (any? myPlayers with [boolSelected AND (who = 0)])[
        set myAskers "THIS SHOULD NOT HAPPEN, HUMAN PLAYER SHOULD NOT BE ASKED"
      ][
        set myAskers "themselves"
      ]
   ]
  report myAskers
end



;;**********************************************************************
;; Output
;;**********************************************************************
;;Current list of coalitions
to CurrentCoalitionsUpdate
  let l []

  ;; Make list of all agents coalitions
  foreach (range 0  myNumAgents)[i ->
    ask myplayer i [set l lput intMyCoalition l ]
  ]

  ;; Make list of different numbers used
  let k remove-duplicates l
  ;; print (word "list of coalitions " l)
  ;; print (word "list of numbers " k)

  ;; we use the position in k as the new coalition number
  ;; player 0 should always be in coalition 0
  foreach (range 0  (length l)) [ i ->
    set l (replace-item i l (position (item i l) k))
  ]

  foreach (range 0  myNumAgents)[i ->
    ask myplayer i [set intMyCoalition (item i l)]
  ]

  ;;report l
end

;;Suggested Coalition as a word (zeros and ones)
to-report MyCurrentCoalitionsOutput
  let l ""
  CurrentCoalitionsUpdate

  ;; need to make sure in order
  foreach (range 0  myNumAgents) [ i ->
    set l (word l ([intMyCoalition] of myplayer i) ", ")
  ]
  report l
end

;;Suggested Coalition as a word (zeros and ones)
to-report MyCurrentSuggestionOutput
  let l ""

  ;;need to make sure in order
 foreach (range 0  myNumAgents)[ i ->
    ifelse ([boolSelected] of myPlayer i) [
      set l (word l 1 ", ")
    ][
     set l (word l 0 ", ")
  ]]
  report l
end

;; Sets up outputter
to MySetupOutput
  ;; output file

  ;; if empty? RunID [set RunID 0]
  let myfileName RunID
  set myfileName (word myfileName "_abmscore")
  set myfileName (word myfileName "_" (substring date-and-time 16 25) "_")
  set myfileName (word myfileName (substring date-and-time 0 2))
  set myfileName (word myfileName (substring date-and-time 3 5))
  set myfileName (word myfileName (substring date-and-time 6 8))
  set myfileName (word myfileName (substring date-and-time 13 15))

  set myFileName (word myfileName   ".csv")
  print myFileName
  file-open myFileName
  file-print "Game, Player_turn, Consistant, Player_decision"


end

to-report MyOutPutlinePart1 [boolPlayerTurn boolPlayerAccept]
  ;; outputs line from run, does everything up to to current situation

  let l (word gamenumber ", " )
 ;; 1) Player or bot turn
  ifelse boolPlayerTurn [set l (word l "1, ")][set l (word l "0, ")]

 ;; 2) Player action consistant with bots
  set l (word l (BotDoSame boolPlayerTurn boolPlayerAccept) ", ")
  ;; 3) Player decision if bot turn (zero otherwise)
  ifelse boolPlayerTurn [
    ;; So it was the players turn, was it possible to make that selection
    set l (word l CouldBotMakeSelection ", ")
  ][
    ifelse boolPlayerAccept [set l (word l "1, ")][set l (word l "0, ")]
  ]

   ;; 4) current coalitions (in special form)
   set l (word l ", " MyCurrentCoalitionsOutput)

 ;; 5) suggest coalition
   set l (word l ", " MyCurrentSuggestionOutput)


  MyErrorPrint l
  report l

end

to MyOutputlinePart2 [l]
  ;; Does everything post the update

  file-print (word l ", " MyStdOutputline)

end


to MyFinalOutPutline
  ;; Use 2 to indicate final
   let l (word gamenumber ", 2, 2, 2, " )

   ;; 4) current coalitions (in special form)
   set l (word l ", " MyCurrentCoalitionsOutput)

 ;; 5) suggest coalition
   set l (word l ", " MyCurrentSuggestionOutput)

  set l (word l ", " MyStdOutputline)

  MyErrorPrint l
  file-print l

end

to-report MyStdOutputline
 ;; 5 Updated coalitions (in special form)
   let l MyCurrentCoalitionsOutput

  ;; 6) Round number
  set l (word l ", " myRound)


  ;; 7) in core
  set l (word l ", , " MyCSInCore)


  ;; 8) Player 0 in a coalition that is in a core
  set l (word l ", , " MyPlayerInCore)

  ;; 7) Payoff of Player 0
  set l (word l ", , " [dblCurrentValue] of myPlayer 0)

  report l
end

;;**********************************************************************
;; Comparison
;;**********************************************************************
to-report BotDoSame [boolPlayerTurn boolPlayerAccept]
  ;; So this check if bots would do the same
  ifelse boolPlayerTurn [ report myBotOkWithSelection][report myBotOkWithSuggestions(boolPlayerAccept)]
end

to-report myBotOkWithSelection
  ;; So checking that bots would have made same selection
  ;; This needs to happen before code is cleared
    let boolMakeSense CheckHumanChoose SelectedCoalitionValue
    if (myErrorCheck AND (not boolMakeSense)) [print "ERROR CHECK: Human player should not have done that"]

  ifelse boolMakeSense [report 1][report 0]

end

to-report myBotOkWithSuggestions[boolPlayerAccept]
  ;; So checking that bots made same yes/no decision onwould have made same suggestion
  ;;
  let boolHumanShouldHave CheckHumanChoose SelectedCoalitionValue
  let boolMakeSense TRUE
  let boolMistake FALSE

  if not (boolPlayerAccept = boolHumanShouldHave)[
    set boolMakeSense FALSE
    set boolMistake TRUE
  ]

  if (myErrorCheck AND boolMistake)[
        ifelse boolPlayerAccept [
          MyErrorPrint("Human should not have accepted")
        ][
          MyErrorPrint("Human should have accepted")
        ]
  ]

  ifelse boolMakeSense [report 1][report 0]
end

to-report CouldBotMakeSelection
;; This function checks to see if algorithm could have made players selection
  ;; need count number of coalitions involved
  let l []
  ;; Make list of all agents coalitions
  ask myplayers with [boolselected] [
    set l lput intMyCoalition l
  ]

  ;; Make list of different numbers used
  set l remove-duplicates l

  let n length l
  let boolBotCould TRUE
  (ifelse
    n > 2 [
      set boolBotCould FALSE
      MyErrorPrint("N.B. three coalitions considered")
    ]
    n = 2 [
      foreach l[i ->
        MyErrorPrint("N.B. two coalitions considered")
        let c (count myPlayers with [(intMyCoalition = i) AND boolSelected])

        if ((c > 1) AND ((count myPlayers with [intMyCoalition = i]) > (count myPlayers with [(intMyCoalition = i) AND boolSelected] )))
        [
         set boolBotCould FALSE
         MyErrorPrint("N.B. player choose a proper subgroup that is not singleton, and they choose two coalitions to consider")
        ]
      ]
    ]
    n = 0 [
      MyErrorPrint("Check found no coalition selected")
      set boolBotCould FALSE
    ]
    ;;n == 1 is ok
  )

  ifelse boolBotCould [report 1][report 0]
end

to-report CheckHumanChoose [v]
   let b TRUE
  ask myplayer 0 [
    if v <=  dblCurrentValue [
      set b FALSE
    ]
  ]
  report b
end


;;**********************************************************************
;; Error
;;**********************************************************************
to MyErrorPrint[t]
  if myErrorCheck [
    write "ERROR CHECK: "
    print t
  ]
end

;;**********************************************************************
;; inputs
;;**********************************************************************
to-report MyLeftInput[n]
 report (ifelse-value
      n = 0 [(list 1 2 7 4 7 1 0)]
      n = 1 [(list 0 4)]
      n = 2 [(list 2 1)]
      n = 3 [(list 3 1 4)]
      n = 4 [(list 2 1 4 1 3 0 2)]
      n = 5 [(list 1 4 1 0 2 2 2)]
       [(list 1 1)]
  )
end

to-report MyRightInput[n]
 report (ifelse-value
      n = 0 [(list 5 6 3 8 1 0 0)]
      n = 1 [(list 4 0)]
      n = 2 [(list 1 3)]
      n = 3 [(list 4 3 1)]
      n = 4 [(list 1 2 0 3 2 3 3)]
      n = 5 [(list 3 0 3 4 2 1 0)]
       [(list 1 1)]
  )
end


to-report MyCoreCS[n]
 report (ifelse-value
    ;; So make a list of lists
    n = 0 [[]]
    n = 1 [list (list 0 0) []]
    n = 2 [list (list 0 0) []]
    n = 3 [list (list 0 1 1) []]
    n = 4 [list (list 0 1 0 0 2 0 2) []]
    n = 5 [(list (list 0 0 0 1 2 1 1) (list 0 1 0 1 2 0 0) (list 0 1 2 1 3 2 0) (list 0 1 2 1 3 0 2 ))]
       [(list 1 1)]
  )
end

;;**********************************************************************
;; Core comparison
;;**********************************************************************

to-report MyCSInCore
  let l MyCoreCS(GameNumber)
  let IntInCore -1

  ;; So check every possible core CS
  foreach l [x ->
    if not ((IntInCore >= 0) OR (empty? x))[
      let boolDummy TRUE
      ;; if not correct for one case, then not core
      ask myplayers [
        if Not(item who x = intMyCoalition)[
          set boolDummy FALSE
        ]
      ]
      if boolDummy [set IntInCore (position x l)]
    ]
  ]


  report IntInCore

end

;; This only checks if player in a core coalition
to-report MyPlayerInCore
  let l MyCoreCS(GameNumber)
  let IntInCore -1

  ;; So check every possible core CS
  foreach l [x ->
    if not ((IntInCore >= 0) OR (empty? x))[
      let boolDummy TRUE
      ;; if not correct for one case, then not core
      ask myplayers [
        if (item who x =  0) or (0 = intMyCoalition)[
          ;; So if the player susponded to be Zero coalition or is in zero coalition
        if Not(item who x = intMyCoalition)[
          set boolDummy FALSE
        ]]
      ]
      if boolDummy [set IntInCore (position x l)]
    ]
  ]


  report IntInCore

end
@#$#@#$#@
GRAPHICS-WINDOW
5
10
841
450
-1
-1
33.16
1
35
1
1
1
0
1
1
1
0
24
0
12
0
0
1
ticks
10.0

BUTTON
882
17
946
50
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
883
73
946
106
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
870
353
1017
386
Happy with Selection
set mySelection TRUE\nPrint \"Player made coalition selection.\"\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
872
408
935
441
Finish
print \"***** Player finish with run *****\"\nset myEnd TRUE\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

TEXTBOX
885
129
1035
171
Press Setup and then Go.\nThe Go buttom should remain indented.
11
0.0
1

CHOOSER
872
194
1010
239
GameNumber
GameNumber
0 1 2 3 4 5
5

SWITCH
1019
22
1109
55
AutoRun
AutoRun
1
1
-1000

TEXTBOX
1062
322
1212
340
BOT MODE ONLY
11
0.0
1

INPUTBOX
1057
349
1212
409
AutoRunNumber
1000.0
1
0
Number

INPUTBOX
1076
438
1231
498
RunID
1000.0
1
0
Number

@#$#@#$#@
## WHAT IS IT?
The purpose of the model is to collect information on human decision-making in the context of coalition formation games. The model uses a human-in-the-loop approach, and a single human is involved in each trial. All other agents are controlled by the ABMSCORE algorithm (Vernon-Bido and Collins 2020), which is an extension of the algorithm created by Collins and Frydenlund (2018). The glove game, a standard cooperative game, is used as the model scenario. 

The intent of the game is to collection information on the human players behavior and how that compares to the computerized agents behavior. The final coalition structure of the game is compared to an ideal output (the core of the games).

## HOW IT WORKS

The human player plays player '0.' As the game progresses, they get to suggest coalitions to the other players and will have to decide to join coalitions suggesed to them. All players are trying to maximum their payoff by selling pairs of gloves. All gloves in a coalition are pooled, and the revenue generated is equally split.

Each round of the game, the human player gets to suggest a coalition to the other players. If all effected other players like this suggesed coalition, then it forms.

The output of the model is CSV file, which captures alls the decisions the human player had to make during the game, the coalitions suggested, the current coalition structure, the players current reward, and how the coalition compares to an ideal (core partition).

## HOW TO USE IT
First, the game the you wish to play should be selected using GameNumber. Games 1 to 3 are for training, game 4 and 5 are complex. Game 0 should be ignored as it is for testing purposes only.

Once the choosen game is selected, press Setup which will show a visual of the all the players and their glove endownments. The colors of the agents indicate the current coalition they are in. All players start on their own, hence different colors.

To setup the game, press Go. Once go press, the human player is able to make a coalition suggestion. This is done by pressing the players that they which to form a coalition with. 

Once the human player is happy with their selection they press "HappywithSelection." The suggested coalition is proposed to the effected players. If all are happy, then the coalition forms. 

After the human players has made their suggestion, then computerized agents also get to make suggested coalitions. If these suggested coalitions envolve the human player, pop-up questions will appear.

This process is repeated until the number player is happy with coalition or does not what to play anymore. They press Finish to finish the game.

AutoRunNumber, RunID, AutoRun are all used in the automated version of the game and are not discussed here.


## THINGS TO NOTICE
Notice that when a player is selected their shape changes from a cirle to a square. Notice also that a coalitions color might change from round to round.

## THINGS TO TRY
It is suggested that you try games 1 to 3 first to get a clear understanding of the game.

## EXTENDING THE MODEL
Changing the games considered is straight forward if you know netlogo code. You will need to change the values in the functions input section of the code. Recommend that you start by changing game 0. 

## NETLOGO FEATURES
The program uses Netlogo's human interface features. It is basically in a holding pattern until the human does something. As such, you will see the tick increase while it waits.


## RELATED MODELS
N/A

## CREDITS AND REFERENCES

Collins, A.J. and E. Frydenlund, Strategic Group Formation in Agent-based Simulation. Simulation. 94 (2018) 179-193

Vernon-Bido, D. and A.J. Collins, Finding Core Members Of Cooperative Games Using Agent-Based Modeling. 2020, Old Dominion University: Norfolk, VA. https://arxiv.org/abs/2009.00519
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

check
false
0
Polygon -7500403 true true 55 138 22 155 53 196 72 232 91 288 111 272 136 258 147 220 167 174 208 113 280 24 257 7 192 78 151 138 106 213 87 182

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

left glove
true
1
Rectangle -2674135 true true 75 105 225 240
Circle -2674135 true true 75 30 150
Polygon -2674135 true true 225 180 240 150 255 135 270 150 270 180 225 240

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

right glove
true
10
Rectangle -13345367 true true 75 105 225 240
Circle -13345367 true true 75 30 150
Polygon -13345367 true true 75 180 60 150 45 135 30 150 30 180 75 240

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="autoruns4" repetitions="1" runMetricsEveryStep="true">
    <setup>print RunID
setup</setup>
    <go>go</go>
    <final>file-close-all</final>
    <enumeratedValueSet variable="AutoRun">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GameNumber">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="AutoRunNumber">
      <value value="100000"/>
    </enumeratedValueSet>
    <steppedValueSet variable="RunID" first="1" step="1" last="1000"/>
  </experiment>
  <experiment name="autoruns5" repetitions="1" runMetricsEveryStep="true">
    <setup>print RunID
setup</setup>
    <go>go</go>
    <final>file-close-all</final>
    <enumeratedValueSet variable="AutoRun">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GameNumber">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="AutoRunNumber">
      <value value="100000"/>
    </enumeratedValueSet>
    <steppedValueSet variable="RunID" first="1" step="1" last="1000"/>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
